package it.eng.urbanite.dashboard.security;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.keycloak.KeycloakPrincipal;
import org.keycloak.adapters.springsecurity.token.KeycloakAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.oauth2.core.oidc.user.DefaultOidcUser;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationToken;

import it.eng.urbanite.dashboard.config.Constants;

/**
 * Utility class for Spring Security.
 */
public final class SecurityUtils {
	//ONLY FOR DEMO PURPOSE
	public static final Boolean DEGUG_MODE = Boolean.FALSE;
	public static final String  DEGUG_USER = "urbanite";
	
	private SecurityUtils() {}

	/**
	 * Get the login of the current user.
	 * @return the login of the current user.
	 */
	public static Optional<String> getCurrentUserLogin() {
		if (SecurityUtils.DEGUG_MODE.booleanValue()) {
			//String uname = SecurityContextHolder.getContext().getAuthentication().getPrincipal().toString();
			//return Optional.of(uname);
			return Optional.of(DEGUG_USER);
		}
		SecurityContext securityContext = SecurityContextHolder.getContext();
		return Optional.ofNullable(extractPrincipal(securityContext.getAuthentication()));
	}

	/**
	 * Get the authorities of the current user.
	 * @return the authorities of the current user.
	 */
	public static List<String> getCurrentAuthorities() {
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		Stream<String> authorities = getAuthorities(authentication);
		List<String> result = authorities.collect(Collectors.toList());
		return result;
	}

	/**
	 * Get the Custom role codes of the current user. (See picture for Keycloak config)
	 * @return the role codes of the current user.
	 */
	public static List<String> getCurrentRoleCodes() {
		List<String> result = new ArrayList<>();
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		if( authentication != null && authentication instanceof KeycloakAuthenticationToken) {
			KeycloakAuthenticationToken curAuth = (KeycloakAuthenticationToken) authentication;
			KeycloakPrincipal token = (KeycloakPrincipal) curAuth.getPrincipal();
			if (token!=null) {
				Map<String, Object> claimsMap = token.getKeycloakSecurityContext().getToken().getOtherClaims();
				//System.out.println("IDM: claimsMap size is " + claimsMap!=null ? claimsMap.size() : "NULL");
				if (claimsMap!=null) {
					String role_codes = (String) claimsMap.get(Constants.ROLE_CODES);
					//System.out.println("IDM: "+ Constants.ROLE_CODES + " ==> " + role_codes);
					result = buildCodesInList(role_codes);
					//System.out.println("result.size: "+ result.size());
				}
			}
		}
		return result;
	}

	/**
	 * Get the role codes of the current user.
	 * @return the role codes of the current user.
	 */
	public static List<String> getUserRoles() {
		List<String> result = new ArrayList<>();
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		if( authentication != null && authentication instanceof KeycloakAuthenticationToken) {
			KeycloakAuthenticationToken curAuth = (KeycloakAuthenticationToken) authentication;
			KeycloakPrincipal principal = (KeycloakPrincipal) curAuth.getPrincipal();
			if (principal!=null) {
		        Set<String> roles = principal.getKeycloakSecurityContext().getToken().getRealmAccess().getRoles();
		        result = new ArrayList<String>(roles);
			}
		}
		return result;
	}
	
	/**
	 * Transform a String in a list of Strings
	 
	 * @param role_codes Example this "[TRAFFIC,CYCLE,CARS]"
	 * @return List<String>  Example this list of "TRAFFIC","CYCLE","CARS" 
	 */
	private static List<String> buildCodesInList(String role_codes) {
		if (role_codes!=null) {
			role_codes = role_codes.replace("[","");
			role_codes = role_codes.replace("]","");
			String[] vect = role_codes.split(",");
			return Arrays.asList(vect);
		}
		return new ArrayList<>();
	}

//	/**
//	 * Get the Groups code of the current user.
//	 * @return the Groups code of the current user.
//	 */
//	public static List<String> getCurrentGroupsCode() {
//		//DEVELOP GROUP CODE (Keycloak)
//		//String username = getCurrentUserLogin().orElse(Constants.SYSTEM);
//		//HashMap<String, List<UserRepresentation>> map = userService.findGroupsFromUser(username);
//		//return new ArrayList<String>(map.keySet());
//		return new ArrayList<String>();
//	}

	private static String extractPrincipal(Authentication authentication) {
		if (authentication == null) {
			return null;
		} else if (authentication instanceof KeycloakAuthenticationToken) {
			//THIS GET NAME!
			return (String) ((KeycloakAuthenticationToken) authentication).getName();
		} else if (authentication.getPrincipal() instanceof UserDetails) {
			UserDetails springSecurityUser = (UserDetails) authentication.getPrincipal();
			return springSecurityUser.getUsername();
		} else if (authentication instanceof JwtAuthenticationToken) {
			return (String) ((JwtAuthenticationToken) authentication).getToken().getClaims().get("preferred_username");
		} else if (authentication.getPrincipal() instanceof DefaultOidcUser) {
			Map<String, Object> attributes = ((DefaultOidcUser) authentication.getPrincipal()).getAttributes();
			if (attributes.containsKey("preferred_username")) {
				return (String) attributes.get("preferred_username");
			}
		} else if (authentication.getPrincipal() instanceof String) {
			return (String) authentication.getPrincipal();
		}
		return null;
	}

	/**
	 * Check if a user is authenticated.
	 *
	 * @return true if the user is authenticated, false otherwise.
	 */
	public static boolean isAuthenticated() {
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		return authentication != null && getAuthorities(authentication).noneMatch(AuthoritiesConstants.ROLE_ANONYMOUS::equals);
	}

	/**
	 * Checks if the current user has a specific authority.
	 *
	 * @param authority the authority to check.
	 * @return true if the current user has the authority, false otherwise.
	 */
	public static boolean hasCurrentUserThisAuthority(String authority) {
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		return authentication != null && getAuthorities(authentication).anyMatch(authority::equals);
	}

	private static Stream<String> getAuthorities(Authentication authentication) {
//		Collection<? extends GrantedAuthority> authorities = authentication instanceof JwtAuthenticationToken
//		? extractAuthorityFromClaims(((JwtAuthenticationToken) authentication).getToken().getClaims())
//		: authentication.getAuthorities();
//		return authorities.stream().map(GrantedAuthority::getAuthority);
		Collection<? extends GrantedAuthority> authorities = authentication instanceof KeycloakAuthenticationToken
				? ((KeycloakAuthenticationToken) authentication).getAuthorities()
				: authentication.getAuthorities();
		return authorities.stream().map(GrantedAuthority::getAuthority);
	}

	public static List<GrantedAuthority> extractAuthorityFromClaims(Map<String, Object> claims) {
		return mapRolesToGrantedAuthorities(getRolesFromClaims(claims));
	}

	@SuppressWarnings("unchecked")
	private static Collection<String> getRolesFromClaims(Map<String, Object> claims) {
		return (Collection<String>) claims.getOrDefault("groups", claims.getOrDefault("roles", new ArrayList<>()));
	}

	private static List<GrantedAuthority> mapRolesToGrantedAuthorities(Collection<String> roles) {
		return roles.stream().filter(role -> role.startsWith("ROLE_")).map(SimpleGrantedAuthority::new).collect(Collectors.toList());
	}


}
