package it.eng.urbanite.dashboard.service;

import java.util.List;

import javax.persistence.criteria.JoinType;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

// for static metamodels
import it.eng.urbanite.dashboard.model.DashboardComponent;
import it.eng.urbanite.dashboard.model.DashboardComponent_;
import it.eng.urbanite.dashboard.model.DashboardPage_;
import it.eng.urbanite.dashboard.repository.DashboardComponentRepository;
import it.eng.urbanite.dashboard.service.criteria.DashboardComponentCriteria;
import it.eng.urbanite.dashboard.service.dto.DashboardComponentDTO;
import it.eng.urbanite.dashboard.service.mapper.DashboardComponentMapper;
import tech.jhipster.service.QueryService;

@Service
@Transactional(readOnly = true)
public class DashboardComponentQueryService extends QueryService<DashboardComponent> {
    private final Logger log = LoggerFactory.getLogger(DashboardComponentQueryService.class);
    private final DashboardComponentRepository dashboardComponentRepository;
    private final DashboardComponentMapper dashboardComponentMapper;
    
    public DashboardComponentQueryService(
        DashboardComponentRepository dashboardComponentRepository,
        DashboardComponentMapper dashboardComponentMapper
    ) {
        this.dashboardComponentRepository = dashboardComponentRepository;
        this.dashboardComponentMapper = dashboardComponentMapper;
    }

    /**
     * Return a {@link List} of {@link DashboardComponentDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public List<DashboardComponentDTO> findByCriteria(DashboardComponentCriteria criteria) {
        log.debug("find by criteria : {}", criteria);
        final Specification<DashboardComponent> specification = createSpecification(criteria);
        return dashboardComponentMapper.toDto(dashboardComponentRepository.findAll(specification));
    }

    /**
     * Return a {@link Page} of {@link DashboardComponentDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @param page The page, which should be returned.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public Page<DashboardComponentDTO> findByCriteria(DashboardComponentCriteria criteria, Pageable page) {
        log.debug("find by criteria : {}, page: {}", criteria, page);
        final Specification<DashboardComponent> specification = createSpecification(criteria);
        return dashboardComponentRepository.findAll(specification, page).map(dashboardComponentMapper::toDto);
    }

    /**
     * Return the number of matching entities in the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the number of matching entities.
     */
    @Transactional(readOnly = true)
    public long countByCriteria(DashboardComponentCriteria criteria) {
        log.debug("count by criteria : {}", criteria);
        final Specification<DashboardComponent> specification = createSpecification(criteria);
        return dashboardComponentRepository.count(specification);
    }

    /**
     * Function to convert {@link DashboardComponentCriteria} to a {@link Specification}
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching {@link Specification} of the entity.
     */
    protected Specification<DashboardComponent> createSpecification(DashboardComponentCriteria criteria) {
        Specification<DashboardComponent> specification = Specification.where(null);
        if (criteria != null) {
            if (criteria.getId() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getId(), DashboardComponent_.id));
            }
            if (criteria.getComponentType() != null) {
                specification = specification.and(buildSpecification(criteria.getComponentType(), DashboardComponent_.componentType));
            }
            if (criteria.getComponentName() != null) {
                specification = specification.and(buildStringSpecification(criteria.getComponentName(), DashboardComponent_.componentName));
            }
            if (criteria.getComponentcontent() != null) {
                specification =
                    specification.and(buildStringSpecification(criteria.getComponentcontent(), DashboardComponent_.componentcontent));
            }
            if (criteria.getComponentPosition() != null) {
                specification =
                    specification.and(buildStringSpecification(criteria.getComponentPosition(), DashboardComponent_.componentPosition));
            }
            if (criteria.getCreatedDate() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getCreatedDate(), DashboardComponent_.createdDate));
            }
            if (criteria.getCreatedBy() != null) {
                specification = specification.and(buildStringSpecification(criteria.getCreatedBy(), DashboardComponent_.createdBy));
            }
            if (criteria.getDashboardPageId() != null) {
                specification =
                    specification.and(
                        buildSpecification(
                            criteria.getDashboardPageId(),
                            root -> root.join(DashboardComponent_.dashboardPage, JoinType.LEFT).get(DashboardPage_.id)
                        )
                    );
            }
        }
        return specification;
    }
}
