package it.eng.urbanite.dashboard.service;

import java.util.List;
import java.util.Optional;

import javax.persistence.criteria.JoinType;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.eng.urbanite.dashboard.config.Constants;
// for static metamodels
import it.eng.urbanite.dashboard.model.DashboardComponent_;
import it.eng.urbanite.dashboard.model.DashboardPage;
import it.eng.urbanite.dashboard.model.DashboardPage_;
import it.eng.urbanite.dashboard.model.Target_;
import it.eng.urbanite.dashboard.repository.DashboardPageRepository;
import it.eng.urbanite.dashboard.repository.MenuItemRepository;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.criteria.DashboardPageCriteria;
import it.eng.urbanite.dashboard.service.dto.DashboardPageDTO;
import it.eng.urbanite.dashboard.service.mapper.DashboardPageMapper;
import it.eng.urbanite.dashboard.service.mapper.MenuItemMapper;
import tech.jhipster.service.QueryService;
import tech.jhipster.service.filter.StringFilter;

@Service
@Transactional(readOnly = true)
public class DashboardPageQueryService extends QueryService<DashboardPage> {
    private final Logger log = LoggerFactory.getLogger(DashboardPageQueryService.class);
    private final DashboardPageRepository dashboardPageRepository;
    private final DashboardPageMapper dashboardPageMapper;
    
	private boolean securityEnabled = true;
	
    public DashboardPageQueryService(DashboardPageRepository dashboardPageRepository, 
							    	 DashboardPageMapper dashboardPageMapper,
							    	 MenuItemRepository menuRepository,
							    	 MenuItemMapper menuMapper) {
        this.dashboardPageRepository = dashboardPageRepository;
        this.dashboardPageMapper = dashboardPageMapper;
    }

    /**
     * Return a {@link List} of {@link DashboardPageDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public List<DashboardPageDTO> findByCriteria(DashboardPageCriteria criteria) {
    	
    	log.info("find by criteria : {}", criteria);
        final Specification<DashboardPage> specification = createSpecification(criteria);
        List<DashboardPage> l = dashboardPageRepository.findAll(specification);
		List<DashboardPageDTO> r = dashboardPageMapper.toDto(l);		
		
		String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
		
		r.stream().forEach(p -> p.setPageOwner(createdBy.equals(p.getCreatedBy()))); 
		
		return r;
    }

    /**
     * Return a {@link Page} of {@link DashboardPageDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @param page The page, which should be returned.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public Page<DashboardPageDTO> findByCriteria(DashboardPageCriteria criteria, Pageable page) {
        log.debug("find by criteria : {}, page: {}", criteria, page);
        final Specification<DashboardPage> specification = createSpecification(criteria);
        return dashboardPageRepository.findAll(specification, page).map(dashboardPageMapper::toDto);
    }

    /**
     * Return the number of matching entities in the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the number of matching entities.
     */
    @Transactional(readOnly = true)
    public long countByCriteria(DashboardPageCriteria criteria) {
        log.debug("count by criteria : {}", criteria);
        final Specification<DashboardPage> specification = createSpecification(criteria);
        return dashboardPageRepository.count(specification);
    }

    /**
     * Function to convert {@link DashboardPageCriteria} to a {@link Specification}
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching {@link Specification} of the entity.
     */
    protected Specification<DashboardPage> createSpecification(DashboardPageCriteria criteria) {
        Specification<DashboardPage> specification = Specification.where(null);
        
        if (criteria != null) {

    		if (securityEnabled && criteria.getMypages() != null && criteria.getMypages().getEquals()!= null && criteria.getMypages().getEquals().booleanValue()) {
    			boolean isAutenticated = true; //SecurityUtils.isAuthenticated(); //TO BE TESTED/FIXED!!!
    			//boolean isAdmin = SecurityUtils.isCurrentUserInRole(AuthoritiesConstants.ADMIN);
    			//boolean isUser = SecurityUtils.isCurrentUserInRole(AuthoritiesConstants.USER);
    			
    			Optional<String> loggedUser = SecurityUtils.getCurrentUserLogin();
    			String userName = loggedUser.orElse(null);
    			
    		  //if ( isAutenticated && !isAdmin && !isUser ) {
				if ( isAutenticated ) {
    				StringFilter currentUser = new StringFilter();
    				currentUser.setEquals(userName);
    				specification = specification.and(buildStringSpecification(currentUser, DashboardPage_.createdBy));
    			}
    		  //if ( !isAutenticated || (isAutenticated && !(isAdmin || isUser))) {
    			if ( !isAutenticated ) {
    				StringFilter currentUser = new StringFilter();
    				currentUser.setEquals("-1");
    				specification = specification.and(buildStringSpecification(currentUser, DashboardPage_.createdBy));
    			}
    		}        	
        	
        	
            if (criteria.getId() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getId(), DashboardPage_.id));
            }
            if (criteria.getName() != null) {
                specification = specification.and(buildStringSpecification(criteria.getName(), DashboardPage_.name));
            }
            if (criteria.getDescription() != null) {
                specification = specification.and(buildStringSpecification(criteria.getDescription(), DashboardPage_.description));
            }
            if (criteria.getContent() != null) {
                specification = specification.and(buildStringSpecification(criteria.getContent(), DashboardPage_.content));
            }
            if (criteria.getNote() != null) {
                specification = specification.and(buildStringSpecification(criteria.getNote(), DashboardPage_.note));
            }
            if (criteria.getShared() != null) {
                specification = specification.and(buildSpecification(criteria.getShared(), DashboardPage_.shared));
            }
            if (criteria.getCreatedDate() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getCreatedDate(), DashboardPage_.createdDate));
            }
            if (criteria.getCreatedBy() != null) {
                specification = specification.and(buildStringSpecification(criteria.getCreatedBy(), DashboardPage_.createdBy));
            }
            if (criteria.getDashboardComponentId() != null) {
                specification =
                    specification.and(
                        buildSpecification(
                            criteria.getDashboardComponentId(),
                            root -> root.join(DashboardPage_.dashboardComponents, JoinType.LEFT).get(DashboardComponent_.id)
                        )
                    );
            }
            if (criteria.getTargetId() != null) {
                specification =
                    specification.and(
                        buildSpecification(criteria.getTargetId(), root -> root.join(DashboardPage_.targets, JoinType.LEFT).get(Target_.id))
                    );
            }
        }
        return specification;
    }
}
