package it.eng.urbanite.dashboard.service;

import it.eng.urbanite.dashboard.model.*; // for static metamodels
import it.eng.urbanite.dashboard.model.MenuBlock;
import it.eng.urbanite.dashboard.repository.MenuBlockRepository;
import it.eng.urbanite.dashboard.service.criteria.MenuBlockCriteria;
import it.eng.urbanite.dashboard.service.dto.MenuBlockDTO;
import it.eng.urbanite.dashboard.service.mapper.MenuBlockMapper;
import java.util.List;
import javax.persistence.criteria.JoinType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tech.jhipster.service.QueryService;

@Service
@Transactional(readOnly = true)
public class MenuBlockQueryService extends QueryService<MenuBlock> {
    private final Logger log = LoggerFactory.getLogger(MenuBlockQueryService.class);
    private final MenuBlockRepository menuBlockRepository;
    private final MenuBlockMapper menuBlockMapper;
    
    public MenuBlockQueryService(MenuBlockRepository menuBlockRepository, MenuBlockMapper menuBlockMapper) {
        this.menuBlockRepository = menuBlockRepository;
        this.menuBlockMapper = menuBlockMapper;
    }

    /**
     * Return a {@link List} of {@link MenuBlockDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public List<MenuBlockDTO> findByCriteria(MenuBlockCriteria criteria) {
        log.debug("find by criteria : {}", criteria);
        final Specification<MenuBlock> specification = createSpecification(criteria);
        return menuBlockMapper.toDto(menuBlockRepository.findAll(specification));
    }

    /**
     * Return a {@link Page} of {@link MenuBlockDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @param page The page, which should be returned.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public Page<MenuBlockDTO> findByCriteria(MenuBlockCriteria criteria, Pageable page) {
        log.debug("find by criteria : {}, page: {}", criteria, page);
        final Specification<MenuBlock> specification = createSpecification(criteria);
        return menuBlockRepository.findAll(specification, page).map(menuBlockMapper::toDto);
    }

    /**
     * Return the number of matching entities in the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the number of matching entities.
     */
    @Transactional(readOnly = true)
    public long countByCriteria(MenuBlockCriteria criteria) {
        log.debug("count by criteria : {}", criteria);
        final Specification<MenuBlock> specification = createSpecification(criteria);
        return menuBlockRepository.count(specification);
    }

    /**
     * Function to convert {@link MenuBlockCriteria} to a {@link Specification}
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching {@link Specification} of the entity.
     */
    protected Specification<MenuBlock> createSpecification(MenuBlockCriteria criteria) {
        Specification<MenuBlock> specification = Specification.where(null);
        if (criteria != null) {
            if (criteria.getId() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getId(), MenuBlock_.id));
            }
            if (criteria.getCode() != null) {
                specification = specification.and(buildStringSpecification(criteria.getCode(), MenuBlock_.code));
            }
            if (criteria.getLabel() != null) {
                specification = specification.and(buildStringSpecification(criteria.getLabel(), MenuBlock_.label));
            }
            if (criteria.getOrder() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getOrder(), MenuBlock_.order));
            }
            if (criteria.getType() != null) {
                specification = specification.and(buildSpecification(criteria.getType(), MenuBlock_.type));
            }
            if (criteria.getCreatedDate() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getCreatedDate(), MenuBlock_.createdDate));
            }
            if (criteria.getCreatedBy() != null) {
                specification = specification.and(buildStringSpecification(criteria.getCreatedBy(), MenuBlock_.createdBy));
            }
            if (criteria.getMenuItemId() != null) {
                specification =
                    specification.and(
                        buildSpecification(
                            criteria.getMenuItemId(),
                            root -> root.join(MenuBlock_.menuItems, JoinType.LEFT).get(MenuItem_.id)
                        )
                    );
            }
        }
        return specification;
    }
}
