package it.eng.urbanite.dashboard.service;

import it.eng.urbanite.dashboard.model.*; // for static metamodels
import it.eng.urbanite.dashboard.model.MenuItem;
import it.eng.urbanite.dashboard.repository.MenuItemRepository;
import it.eng.urbanite.dashboard.service.criteria.MenuItemCriteria;
import it.eng.urbanite.dashboard.service.dto.MenuItemDTO;
import it.eng.urbanite.dashboard.service.mapper.MenuItemMapper;
import java.util.List;
import javax.persistence.criteria.JoinType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tech.jhipster.service.QueryService;

@Service
@Transactional(readOnly = true)
public class MenuItemQueryService extends QueryService<MenuItem> {
    private final Logger log = LoggerFactory.getLogger(MenuItemQueryService.class);
    private final MenuItemRepository menuItemRepository;
    private final MenuItemMapper menuItemMapper;
    
    public MenuItemQueryService(MenuItemRepository menuItemRepository, MenuItemMapper menuItemMapper) {
        this.menuItemRepository = menuItemRepository;
        this.menuItemMapper = menuItemMapper;
    }

    /**
     * Return a {@link List} of {@link MenuItemDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public List<MenuItemDTO> findByCriteria(MenuItemCriteria criteria) {
        log.debug("find by criteria : {}", criteria);
        final Specification<MenuItem> specification = createSpecification(criteria);
        return menuItemMapper.toDto(menuItemRepository.findAll(specification));
    }

    /**
     * Return a {@link Page} of {@link MenuItemDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @param page The page, which should be returned.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public Page<MenuItemDTO> findByCriteria(MenuItemCriteria criteria, Pageable page) {
        log.debug("find by criteria : {}, page: {}", criteria, page);
        final Specification<MenuItem> specification = createSpecification(criteria);
        return menuItemRepository.findAll(specification, page).map(menuItemMapper::toDto);
    }

    /**
     * Return the number of matching entities in the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the number of matching entities.
     */
    @Transactional(readOnly = true)
    public long countByCriteria(MenuItemCriteria criteria) {
        log.debug("count by criteria : {}", criteria);
        final Specification<MenuItem> specification = createSpecification(criteria);
        return menuItemRepository.count(specification);
    }

    /**
     * Function to convert {@link MenuItemCriteria} to a {@link Specification}
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching {@link Specification} of the entity.
     */
    protected Specification<MenuItem> createSpecification(MenuItemCriteria criteria) {
        Specification<MenuItem> specification = Specification.where(null);
        if (criteria != null) {
            if (criteria.getId() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getId(), MenuItem_.id));
            }
            if (criteria.getCode() != null) {
                specification = specification.and(buildStringSpecification(criteria.getCode(), MenuItem_.code));
            }
            if (criteria.getLabel() != null) {
                specification = specification.and(buildStringSpecification(criteria.getLabel(), MenuItem_.label));
            }
            if (criteria.getOrder() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getOrder(), MenuItem_.order));
            }
            if (criteria.getType() != null) {
                specification = specification.and(buildSpecification(criteria.getType(), MenuItem_.type));
            }
            if (criteria.getCreatedDate() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getCreatedDate(), MenuItem_.createdDate));
            }
            if (criteria.getCreatedBy() != null) {
                specification = specification.and(buildStringSpecification(criteria.getCreatedBy(), MenuItem_.createdBy));
            }
            if (criteria.getDashboardPageId() != null) {
                specification =
                    specification.and(
                        buildSpecification(
                            criteria.getDashboardPageId(),
                            root -> root.join(MenuItem_.dashboardPage, JoinType.LEFT).get(DashboardPage_.id)
                        )
                    );
            }
            if (criteria.getMenuBlockId() != null) {
                specification =
                    specification.and(
                        buildSpecification(
                            criteria.getMenuBlockId(),
                            root -> root.join(MenuItem_.menuBlock, JoinType.LEFT).get(MenuBlock_.id)
                        )
                    );
            }
        }
        return specification;
    }
}
