package it.eng.urbanite.dashboard.service;

import java.util.List;

import javax.persistence.criteria.JoinType;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

// for static metamodels
import it.eng.urbanite.dashboard.model.DashboardPage_;
import it.eng.urbanite.dashboard.model.Target;
import it.eng.urbanite.dashboard.model.Target_;
import it.eng.urbanite.dashboard.repository.TargetRepository;
import it.eng.urbanite.dashboard.service.criteria.TargetCriteria;
import it.eng.urbanite.dashboard.service.dto.TargetDTO;
import it.eng.urbanite.dashboard.service.mapper.TargetMapper;
import tech.jhipster.service.QueryService;

@Service
@Transactional(readOnly = true)
public class TargetQueryService extends QueryService<Target> {
    private final Logger log = LoggerFactory.getLogger(TargetQueryService.class);
    private final TargetRepository targetRepository;
    private final TargetMapper targetMapper;
    
    public TargetQueryService(TargetRepository targetRepository, TargetMapper targetMapper) {
        this.targetRepository = targetRepository;
        this.targetMapper = targetMapper;
    }

    /**
     * Return a {@link List} of {@link TargetDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public List<TargetDTO> findByCriteria(TargetCriteria criteria) {
        log.debug("find by criteria : {}", criteria);
        final Specification<Target> specification = createSpecification(criteria);
        return targetMapper.toDto(targetRepository.findAll(specification));
    }

    /**
     * Return a {@link Page} of {@link TargetDTO} which matches the criteria from the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @param page The page, which should be returned.
     * @return the matching entities.
     */
    @Transactional(readOnly = true)
    public Page<TargetDTO> findByCriteria(TargetCriteria criteria, Pageable page) {
        log.debug("find by criteria : {}, page: {}", criteria, page);
        final Specification<Target> specification = createSpecification(criteria);
        return targetRepository.findAll(specification, page).map(targetMapper::toDto);
    }

    /**
     * Return the number of matching entities in the database.
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the number of matching entities.
     */
    @Transactional(readOnly = true)
    public long countByCriteria(TargetCriteria criteria) {
        log.debug("count by criteria : {}", criteria);
        final Specification<Target> specification = createSpecification(criteria);
        return targetRepository.count(specification);
    }

    /**
     * Function to convert {@link TargetCriteria} to a {@link Specification}
     * @param criteria The object which holds all the filters, which the entities should match.
     * @return the matching {@link Specification} of the entity.
     */
    protected Specification<Target> createSpecification(TargetCriteria criteria) {
        Specification<Target> specification = Specification.where(null);
        if (criteria != null) {
            if (criteria.getId() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getId(), Target_.id));
            }
            if (criteria.getType() != null) {
                specification = specification.and(buildSpecification(criteria.getType(), Target_.type));
            }
            if (criteria.getName() != null) {
                specification = specification.and(buildStringSpecification(criteria.getName(), Target_.name));
            }
            if (criteria.getNote() != null) {
                specification = specification.and(buildStringSpecification(criteria.getNote(), Target_.note));
            }
            if (criteria.getCreatedDate() != null) {
                specification = specification.and(buildRangeSpecification(criteria.getCreatedDate(), Target_.createdDate));
            }
            if (criteria.getCreatedBy() != null) {
                specification = specification.and(buildStringSpecification(criteria.getCreatedBy(), Target_.createdBy));
            }
            if (criteria.getDashboardPageId() != null) {
                specification =
                    specification.and(
                        buildSpecification(
                            criteria.getDashboardPageId(),
                            root -> root.join(Target_.dashboardPage, JoinType.LEFT).get(DashboardPage_.id)
                        )
                    );
            }
        }
        return specification;
    }
}
