package it.eng.urbanite.dashboard.service.impl;

import java.time.Instant;
import java.util.List;
import java.util.Optional;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.eng.urbanite.dashboard.config.Constants;
import it.eng.urbanite.dashboard.model.DashboardComponent;
import it.eng.urbanite.dashboard.model.DashboardPage;
import it.eng.urbanite.dashboard.model.MenuItem;
import it.eng.urbanite.dashboard.model.Target;
import it.eng.urbanite.dashboard.model.enumeration.CloningOpionType;
import it.eng.urbanite.dashboard.repository.DashboardComponentRepository;
import it.eng.urbanite.dashboard.repository.DashboardPageRepository;
import it.eng.urbanite.dashboard.repository.MenuItemRepository;
import it.eng.urbanite.dashboard.repository.TargetRepository;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.DashboardPageService;
import it.eng.urbanite.dashboard.service.dto.DashboardComponentDTO;
import it.eng.urbanite.dashboard.service.dto.DashboardPageDTO;
import it.eng.urbanite.dashboard.service.dto.TargetDTO;
import it.eng.urbanite.dashboard.service.mapper.DashboardComponentMapper;
import it.eng.urbanite.dashboard.service.mapper.DashboardPageMapper;
import it.eng.urbanite.dashboard.service.mapper.MenuItemMapper;
import it.eng.urbanite.dashboard.service.mapper.TargetMapper;

/**
 * Service Implementation for managing {@link DashboardPage}.
 */
@Service
@Transactional
public class DashboardPageServiceImpl implements DashboardPageService {
    private final Logger log = LoggerFactory.getLogger(DashboardPageServiceImpl.class);
   
    private final DashboardPageRepository dashboardPageRepository;
    private final DashboardPageMapper dashboardPageMapper;

    private final TargetRepository targetRepository;
    private final TargetMapper targetMapper;

    private final DashboardComponentRepository dashboardComponentRepository;
    private final DashboardComponentMapper dashboardComponentMapper;

    private final MenuItemRepository menuItemRepository;
	
    public DashboardPageServiceImpl(DashboardPageRepository dashboardPageRepository, 
    								DashboardPageMapper dashboardPageMapper, 
    								TargetRepository targetRepository, 
    								TargetMapper targetMapper,
    								DashboardComponentRepository dashboardComponentRepository,
    								DashboardComponentMapper dashboardComponentMapper,
    								MenuItemRepository menuItemRepository,
    								MenuItemMapper menuItemMapper
    		) {
        this.dashboardPageRepository = dashboardPageRepository;
        this.dashboardPageMapper = dashboardPageMapper;
        this.targetRepository = targetRepository;
        this.targetMapper = targetMapper;
        this.dashboardComponentRepository = dashboardComponentRepository;
        this.dashboardComponentMapper= dashboardComponentMapper;
        this.menuItemRepository = menuItemRepository;
    }

    @Override
    public DashboardPageDTO save(DashboardPageDTO dashboardPageDTO) {
    	log.debug("Request to save DashboardPage : {}", dashboardPageDTO);
    	DashboardPage dashboardPage = dashboardPageMapper.toEntity(dashboardPageDTO);
    	dashboardPage = dashboardPageRepository.save(dashboardPage);
    	return dashboardPageMapper.toDto(dashboardPage);
    }

    @Override
    @Transactional(readOnly = true)
    public Page<DashboardPageDTO> findAll(Pageable pageable) {
        log.debug("Request to get all DashboardPages");
        return dashboardPageRepository.findAll(pageable).map(dashboardPageMapper::toDto);
    }

    @Override
    @Transactional(readOnly = true)
    public Optional<DashboardPageDTO> findOne(Long id) {
        log.debug("Request to get DashboardPage : {}", id);
        return dashboardPageRepository.findById(id).map(dashboardPageMapper::toDto);
    }

    @Override
    public void delete(Long id) {
        log.debug("Request to delete DashboardPage : {}", id);
        dashboardPageRepository.deleteById(id);
    }

	@Override
	public List<DashboardPageDTO> findAll() {
		log.debug("Request to get all DashboardPages");
		List<DashboardPage> l = dashboardPageRepository.findAll();
		List<DashboardPageDTO> r = dashboardPageMapper.toDto(l);
		
		//Set Boolean PageOwner: True when Current user created the page
		String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
		r.stream().forEach( p -> p.setPageOwner(createdBy.equals(p.getCreatedBy())));
		
        return r;
	}

	@Override
	public List<DashboardPageDTO> findAllPagesShared(String createdBy, Boolean shared, List<String> roles, List<String> groups) {
		log.debug("Request to get shared DashboardPages:  {}  {}  {}  {}", createdBy, shared, roles, groups);
		List<DashboardPage> l = dashboardPageRepository.findAllPagesShared(createdBy, shared, roles, groups);
		List<DashboardPageDTO> r = dashboardPageMapper.toDto(l);
		
		//Set Boolean PageOwner: True when Current user created the page
		//r.stream().forEach( p -> p.setPageOwner(createdBy.equals(p.getCreatedBy())));
		for(DashboardPageDTO p : r) {
			p.setPageOwner(createdBy.equals(p.getCreatedBy()));
			List<MenuItem> listTypes = menuItemRepository.findByDashboardPageId(p.getId());
			p.setType(listTypes!=null && listTypes.size()>0 ? listTypes.get(0).getType(): null);
		}
		
        return r;
	}
	
	@Override
	public List<DashboardPageDTO> findAllPagesPersonal(String createdBy) {
		log.debug("Request to get shared DashboardPages:  {} ", createdBy);
		List<DashboardPage> l = dashboardPageRepository.findAllPagesPersonal(createdBy);
		List<DashboardPageDTO> r = dashboardPageMapper.toDto(l);
		
		//Set Boolean PageOwner: True when Current user created the page
		//r.stream().forEach( p -> p.setPageOwner(createdBy.equals(p.getCreatedBy())));
		for(DashboardPageDTO p : r) {
			p.setPageOwner(createdBy.equals(p.getCreatedBy()));
			List<MenuItem> listTypes = menuItemRepository.findByDashboardPageId(p.getId());
			p.setType(listTypes!=null && listTypes.size()>0 ? listTypes.get(0).getType(): null);
		}
		
		return r;
	}

	@Override
	public DashboardPageDTO duplicatePage(Long id, String createdBy, String dashboardName, CloningOpionType option) {
		log.info("Duplicate the DashboardPage with id {}", id);
		
		//Retrieve Page
		Instant now = Instant.now();
		log.info("Duplicating DashboardPage with id {}: 1.Retrieve Page", id);
		DashboardPage dashboardPage = dashboardPageRepository.findById(id).orElse(null);
		DashboardPageDTO dashboardPageDTO = dashboardPageMapper.toDto(dashboardPage);
		
		//Clone Page
		log.info("Duplicating DashboardPage with id {}: 2.Clone Page", id);
		dashboardPageDTO.setId(null);
		dashboardPageDTO.setCreatedBy(createdBy);
		dashboardPageDTO.setCreatedDate(now);
		if (dashboardName!= null ) {
			dashboardPageDTO.setName(dashboardName);
		}
		DashboardPageDTO newDashboardPageDTO = this.save(dashboardPageDTO);
		
		//Clone Targets
		if (CloningOpionType.ALL.equals(option) || CloningOpionType.TARGETS.equals(option)) {
			log.info("Duplicating DashboardPage with id {}: 3.Clone Target", id);
			Set<Target> targets = dashboardPage.getTargets();
			for (Target t: targets) {
				TargetDTO newTargetDTO = targetMapper.toDto(t);
				newTargetDTO.setId(null);
				newTargetDTO.setCreatedBy(createdBy);
				newTargetDTO.setCreatedDate(now);
				newTargetDTO.setDashboardPage(newDashboardPageDTO);
				
				Target newTarget = targetMapper.toEntity(newTargetDTO);
				newTarget = targetRepository.save(newTarget);
			}
		}else {
			log.info("SKIP Duplicating DashboardPage with id {}: 3.Clone Target", id);
		}
		
		//Clone Components
		if (CloningOpionType.ALL.equals(option) || CloningOpionType.COMPONENTS.equals(option)) {
			log.info("Duplicating DashboardPage with id {}: 4.Clone Components", id);
			Set<DashboardComponent> components = dashboardPage.getDashboardComponents();
			for (DashboardComponent c: components) {
				DashboardComponentDTO newComponentDTO = dashboardComponentMapper.toDto(c);
				newComponentDTO.setId(null);
				newComponentDTO.setCreatedBy(createdBy);
				newComponentDTO.setCreatedDate(now);
				newComponentDTO.setDashboardPage(newDashboardPageDTO);
				
				DashboardComponent newComponent = dashboardComponentMapper.toEntity(newComponentDTO);
				newComponent = dashboardComponentRepository.save(newComponent);
			}
		}else {
			log.info("SKIP Duplicating DashboardPage with id {}: 4.Clone Components", id);
		}
		
		//TODO MENU BLCOKS AND ITEMS (if needed)
		
		log.info("Duplicaed DashboardPage with id {}: 5.Cloned !!", id);
		return newDashboardPageDTO;
	}

	@Override
	public void deepDelete(Long id) {
		 log.debug("Request to delete DashboardPage deep : {}", id);
		 DashboardPage page = dashboardPageRepository.findById(id).orElse(null);
		 if(page!=null) {
			 log.debug("Delete targets linked to DashboardPage id : {}", id);
			 for(Target t: page.getTargets()) {
				 targetRepository.deleteById(t.getId());
			 }
			 log.debug("Delete components linked to DashboardPage id : {}", id);
			 for(DashboardComponent c: page.getDashboardComponents()) {
				 targetRepository.deleteById(c.getId());
			 }
			 log.debug("Delete menuItems linked to DashboardPage id : {}", id);
			 for(MenuItem m: menuItemRepository.findByDashboardPage(page)) {
				 menuItemRepository.deleteById(m.getId());
			 }
			 log.debug("Delete finally dashboardPage linked to DashboardPage id : {}", id);
			 dashboardPageRepository.deleteById(id);
			 log.debug("Request to delete DashboardPage deep : {} has been sucessfully done!", id);
		 }
	}
	
}
