package it.eng.urbanite.dashboard.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import org.keycloak.representations.idm.UserRepresentation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.eng.urbanite.dashboard.config.Constants;
import it.eng.urbanite.dashboard.model.MenuBlock;
import it.eng.urbanite.dashboard.model.MenuItem;
import it.eng.urbanite.dashboard.model.enumeration.MenuType;
import it.eng.urbanite.dashboard.repository.MenuBlockRepository;
import it.eng.urbanite.dashboard.repository.MenuItemRepository;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.MenuBlockService;
import it.eng.urbanite.dashboard.service.UserService;
import it.eng.urbanite.dashboard.service.dto.MenuBlockDTO;
import it.eng.urbanite.dashboard.service.dto.MenuItemDTO;
import it.eng.urbanite.dashboard.service.mapper.MenuBlockMapper;
import it.eng.urbanite.dashboard.service.mapper.MenuItemMapper;

/**
 * Service Implementation for managing {@link MenuBlock}.
 */
@Service
@Transactional
public class MenuBlockServiceImpl implements MenuBlockService {
	private final Logger log = LoggerFactory.getLogger(MenuBlockServiceImpl.class);
	private final MenuItemRepository menuItemRepository;
	private final MenuItemMapper menuItemMapper;
	private final MenuBlockRepository menuBlockRepository;
	private final MenuBlockMapper menuBlockMapper;
	private final UserService userService;

	public MenuBlockServiceImpl(MenuItemRepository menuItemRepository, MenuItemMapper menuItemMapper, MenuBlockRepository menuBlockRepository, MenuBlockMapper menuBlockMapper, UserService userService) {
		this.menuItemRepository = menuItemRepository;
		this.menuItemMapper = menuItemMapper;
		this.menuBlockRepository = menuBlockRepository;
		this.menuBlockMapper = menuBlockMapper;
		this.userService = userService;
	}

	@Override
	public MenuBlockDTO save(MenuBlockDTO menuBlockDTO) {
		log.debug("Request to save MenuBlock : {}", menuBlockDTO);
		MenuBlock menuBlock = menuBlockMapper.toEntity(menuBlockDTO);
		menuBlock = menuBlockRepository.save(menuBlock);
		return menuBlockMapper.toDto(menuBlock);
	}

	@Override
	@Transactional(readOnly = true)
	public Page<MenuBlockDTO> findAll(Pageable pageable) {
		log.debug("Request to get all MenuBlocks");
		return menuBlockRepository.findAll(pageable).map(menuBlockMapper::toDto);
	}

	@Override
	@Transactional(readOnly = true)
	public Optional<MenuBlockDTO> findOne(Long id) {
		log.debug("Request to get MenuBlock : {}", id);
		return menuBlockRepository.findById(id).map(menuBlockMapper::toDto);
	}

	@Override
	public void delete(Long id) {
		log.debug("Request to delete MenuBlock : {}", id);
		menuBlockRepository.deleteById(id);
	}

	@Override
	public List<MenuBlockDTO> findAll() {
		log.debug("Request to get all MenuBlocks");
		List<MenuBlock> l = menuBlockRepository.findAll();
		List<MenuBlockDTO> r = menuBlockMapper.toDto(l);
		return r;
	}

	@Override
	public List<MenuBlockDTO> findLinkedItems(MenuType type, String createdBy) {
		log.debug("Request to get Linked all MenuBlocks");
    	//List<String> roles = SecurityUtils.getCurrentRoleCodes();
		List<String> roles = SecurityUtils.getUserRoles(); 
    	
		//MMA - GROUPS (TO BE TESTED)
		//List<String> groups = SecurityUtils.getCurrentGroupsCode();
		HashMap<String, List<UserRepresentation>> map = userService.findGroupsFromUser(createdBy);
		List<String> groups = (map!=null && !map.isEmpty()) ? new ArrayList<String>(map.keySet()): new ArrayList<>();
		
		List<MenuItem> itemList = type==null 
								? menuItemRepository.findAll() 
								: (MenuType.SHARED.equals(type) 
										? menuItemRepository.findMenuItems(createdBy, Boolean.TRUE, MenuType.SHARED.name(), roles, groups ) //Filtered List of (authorized) MenuItem(s)
										: menuItemRepository.findByTypeAndCreatedBy(type, createdBy));
		List<MenuItemDTO> itemListDTO = menuItemMapper.toDto(itemList);
		log.info("## itemListDTO in Total : "+itemListDTO.size());
		
		List<MenuBlock> menu = type==null 
							 ? menuBlockRepository.findAll() 
							 : (MenuType.SHARED.equals(type) 
									 ? menuBlockRepository.findBlocks(createdBy, Boolean.TRUE, MenuType.SHARED.name(), roles, groups )  //Filtered List of (authorized) MenuBlock(s)
									 : menuBlockRepository.findByTypeAndCreatedBy(type, createdBy));
		List<MenuBlockDTO> menuDTOList = menuBlockMapper.toDto(menu);
		log.info("## menuDTOList in Total : "+menuDTOList.size());
		
		for (MenuBlockDTO menuBlockDTO : menuDTOList) {
			//Retrieve all items liked to the menuBlock
			List<MenuItemDTO> linkedItems = itemListDTO.stream().filter(x -> x.getMenuBlock().getId().equals(menuBlockDTO.getId())).collect(Collectors.toList());
			menuBlockDTO.setLinkedItems(linkedItems);
		}
		
		return menuDTOList;
	}

	@Override
	public List<MenuBlockDTO> findEditableBlocks() {
		log.info("Request to get all EDITABLE MenuBlocks ...");
		String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
				
		List<MenuBlock> menu = menuBlockRepository.findEditable(createdBy);
		List<MenuBlockDTO> menuDTOList = menuBlockMapper.toDto(menu);
		log.info("## EDITABLE menuDTOList in Total : "+ menuDTOList.size());

		return menuDTOList;
	}

}
