package it.eng.urbanite.dashboard.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

import org.jboss.resteasy.client.jaxrs.ResteasyClientBuilder;
import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.KeycloakBuilder;
import org.keycloak.representations.idm.GroupRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.keycloak.representations.idm.UserRepresentation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.eng.urbanite.dashboard.service.UserService;
import it.eng.urbanite.dashboard.service.dto.UserDTO;
import it.eng.urbanite.dashboard.service.mapper.UserMapper;

/**
 * Service Implementation for managing Users (IDM)
 */
@Service
@Transactional
public class UserServiceImpl implements UserService {
	private static final int MAX_MEMBERS = 1000;
	private final Logger log = LoggerFactory.getLogger(UserServiceImpl.class);
	private final UserMapper userMapper;

	private static final String ADMIN_CLI = "admin-cli";

	@Value("${keycloak.realm}") private String keycloakRealm;
	@Value("${keycloak.auth-server-url}") private String keycloakServerUrl;

	@Value("${idm.user4view.username}") private String username;
	@Value("${idm.user4view.password}") private String password;


	public UserServiceImpl(UserMapper userMapper) {
		this.userMapper = userMapper;
	}

	@Override
	public List<UserDTO> findAllUsers() {
		log.info("Retrieving users on IDM..");
		List<UserDTO> resultDTO = new ArrayList<>();
		try {
			Keycloak kc = getKeycloakClient();
			//TokenManager tokenManager = kc.tokenManager();
			//AccessTokenResponse accessTokenResponse = tokenManager.getAccessToken();
			//log.debug("## accessTokenResponse ==> " + accessTokenResponse.getToken());
			List<UserRepresentation> result = kc.realm(keycloakRealm).users().list();
			for (UserRepresentation user : result) {
				log.info("### User(IDM)  Username:" + user.getUsername() + "  FirstName:" + user.getFirstName() + "  LastName: " + user.getLastName() + "   Email:"+user.getEmail());
			}
			log.info("Retrieved total " + (result!=null ? result.size(): "-") + " users!");
			resultDTO = userMapper.toDto(result);
		}catch (Exception e) {
			log.error("Retrieving users on IDM failed due to error : "+e.getMessage());
		}
		return resultDTO;
	}


	@Override
	public List<RoleRepresentation> findAllRoles() {
		log.info("Retrieving roles on IDM..");
		List<RoleRepresentation> result = new ArrayList<>();
		try {
			Keycloak kc = getKeycloakClient();
			result = kc.realm(keycloakRealm).roles().list();
			for (RoleRepresentation role : result) {
				log.debug("### Role(IDM)  Name:" + role.getName()+ "  Description:" + role.getDescription() + "  Id: " + role.getId() );
			}
			log.info("Retrieved total " + (result!=null ? result.size(): "-") + " roles!");
		}catch (Exception e) {
			log.error("Retrieving roles on IDM failed due to error : "+e.getMessage());
		}
		return result;
	}

	@Override
	public List<GroupRepresentation> findAllGroups() {
		log.info("Retrieving groups on IDM..");
		List<GroupRepresentation> result = new ArrayList<>();
		try {
			Keycloak kc = getKeycloakClient();
			result = kc.realm(keycloakRealm).groups().groups();
			for (GroupRepresentation group : result) {
				log.debug("### Group(IDM)  Name:" + group.getName()+ "  Id: " + group.getId() );
			}
			log.info("Retrieved total " + (result!=null ? result.size(): "-") + " groups!");
		}catch (Exception e) {
			log.error("Retrieving groups on IDM failed due to error : "+e.getMessage());
		}
		return result;
	}
	
	@Override
	public HashMap<String, List<UserRepresentation>> findGroupsFromUser(String username) {
		log.info("Retrieving user_groups on IDM..");
		HashMap<String, List<UserRepresentation>> result = new HashMap<>();
		try {
			Keycloak kc = getKeycloakClient();
			
			List<GroupRepresentation> groups = kc.realm(keycloakRealm).groups().groups();
			for (GroupRepresentation g : groups) {
				// log.info("### User/Groups(IDM)  groupName:"+g.getName());
				List<UserRepresentation> members = kc.realm(keycloakRealm).groups().group(g.getId()).members(0, MAX_MEMBERS);
				log.info("### User/Groups(IDM)  groupName:"+g.getName() + " members: " + (members!=null ? members.size(): "-") + " members!");
				
				if(username!=null) {
					List<UserRepresentation> filteredMembers = members.stream().filter( obj -> obj.getUsername().equalsIgnoreCase(username)).collect(Collectors.toList());
					if (filteredMembers!=null && filteredMembers.size()>0) {
						result.put(g.getName(), filteredMembers);
					}
				}
				
			}
			
		}catch (Exception e) {
			log.error("Retrieving user_groups on IDM failed due to error : "+e.getMessage());
		}
		return result;
	}
	
	
	
	
	
	
	/**
	 * Create a Keycloak client
	 * @return Keycloak client
	 */
	private Keycloak getKeycloakClient() {
		// See https://stackoverflow.com/questions/48583361/how-can-i-read-all-users-using-keycloak-and-spring
		// Create a Keycloak client
		Keycloak kc = KeycloakBuilder.builder()
				.serverUrl(keycloakServerUrl)
				.realm(keycloakRealm)
				.username(username)
				.password(password)
				.clientId(ADMIN_CLI)
				.resteasyClient(new ResteasyClientBuilder().connectionPoolSize(10).build())
				.build();
		return kc;
	}

}
