package it.eng.urbanite.dashboard.web;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.Instant;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.eng.urbanite.dashboard.config.Constants;
import it.eng.urbanite.dashboard.repository.DashboardComponentRepository;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.DashboardComponentQueryService;
import it.eng.urbanite.dashboard.service.DashboardComponentService;
import it.eng.urbanite.dashboard.service.criteria.DashboardComponentCriteria;
import it.eng.urbanite.dashboard.service.dto.DashboardComponentDTO;
import it.eng.urbanite.dashboard.utils.HeaderUtil;
import it.eng.urbanite.dashboard.web.error.BadRequestAlertException;
import tech.jhipster.service.filter.LongFilter;

/**
 * REST controller for managing {@link it.eng.urbanite.dashboard.domain.DashboardComponent}.
 */
@RestController
@CrossOrigin
@RequestMapping("/api")
public class DashboardComponentResource {
    private final Logger log = LoggerFactory.getLogger(DashboardComponentResource.class);
    private static final String ENTITY_NAME = "dashboardComponent";

    private String applicationName = Constants.APP_NAME;

    private final DashboardComponentService dashboardComponentService;
    private final DashboardComponentRepository dashboardComponentRepository;
    private final DashboardComponentQueryService dashboardComponentQueryService;

    public DashboardComponentResource(
        DashboardComponentService dashboardComponentService,
        DashboardComponentRepository dashboardComponentRepository,
        DashboardComponentQueryService dashboardComponentQueryService
    ) {
        this.dashboardComponentService = dashboardComponentService;
        this.dashboardComponentRepository = dashboardComponentRepository;
        this.dashboardComponentQueryService = dashboardComponentQueryService;
    }

    @PostMapping("/dashboard-components")
    public ResponseEntity<DashboardComponentDTO> createDashboardComponent(@Valid @RequestBody DashboardComponentDTO dashboardComponentDTO)
        throws URISyntaxException {
        log.debug("REST request to save DashboardComponent : {}", dashboardComponentDTO);
        if (dashboardComponentDTO.getId() != null) {
            throw new BadRequestAlertException("A new dashboardComponent cannot already have an ID"+ ENTITY_NAME+ "idexists");
        }
        
        //Fill CreatedBy/CreatedDate
        dashboardComponentDTO.setCreatedBy(SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM));
        dashboardComponentDTO.setCreatedDate(Instant.now());
        
        DashboardComponentDTO result = dashboardComponentService.save(dashboardComponentDTO);
        return ResponseEntity
            .created(new URI("/api/dashboard-components/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    @PutMapping("/dashboard-components/{id}")
    public ResponseEntity<DashboardComponentDTO> updateDashboardComponent(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody DashboardComponentDTO dashboardComponentDTO
    ) throws URISyntaxException {
        log.debug("REST request to update DashboardComponent : {}, {}", id, dashboardComponentDTO);
        if (dashboardComponentDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id "+ ENTITY_NAME+ " idnull");
        }
        if (!Objects.equals(id, dashboardComponentDTO.getId())) {
            throw new BadRequestAlertException("Invalid ID "+ ENTITY_NAME+ " idinvalid");
        }

        if (!dashboardComponentRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found "+ ENTITY_NAME+ " idnotfound");
        }

        DashboardComponentDTO result = dashboardComponentService.save(dashboardComponentDTO);
        return ResponseEntity.ok(result);
    }

    @GetMapping("/dashboard-components")
    public ResponseEntity<List<DashboardComponentDTO>> getAllDashboardComponents() {
    	log.debug("REST request to get DashboardComponents");
        List<DashboardComponentDTO> list = dashboardComponentService.findAll();
        return ResponseEntity.ok().body(list);
    }

    @GetMapping("/dashboard-components-bypage/{pageId}")
    public ResponseEntity<List<DashboardComponentDTO>> getAllDashboardComponentByPageId(@PathVariable Long pageId) {
    	DashboardComponentCriteria criteria = new DashboardComponentCriteria();
    	LongFilter filter = new LongFilter();
    	filter.setEquals(pageId);
    	criteria.setDashboardPageId(filter);
		
        log.debug("REST request to get DashboardComponent by criteria: {}", criteria);
        List<DashboardComponentDTO> list = dashboardComponentQueryService.findByCriteria(criteria);
        log.debug("REST result for DashboardComponent: size is " +  list.size());
        
        return ResponseEntity.ok().body(list);
    }
    
    @GetMapping("/dashboard-components/{id}")
    public ResponseEntity<DashboardComponentDTO> getDashboardComponent(@PathVariable Long id) {
        log.debug("REST request to get DashboardComponent : {}", id);
        Optional<DashboardComponentDTO> dashboardComponentDTO = dashboardComponentService.findOne(id);
        return ResponseEntity.ok().body(dashboardComponentDTO.orElse(null));
    }

    @DeleteMapping("/dashboard-components/{id}")
    public ResponseEntity<Void> deleteDashboardComponent(@PathVariable Long id) {
        log.debug("REST request to delete DashboardComponent : {}", id);
        dashboardComponentService.delete(id);
        return ResponseEntity.noContent().build();
    }
    
}
