package it.eng.urbanite.dashboard.web;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.Instant;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.eng.urbanite.dashboard.config.Constants;
import it.eng.urbanite.dashboard.model.enumeration.CloningOpionType;
import it.eng.urbanite.dashboard.repository.DashboardPageRepository;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.DashboardPageQueryService;
import it.eng.urbanite.dashboard.service.DashboardPageService;
import it.eng.urbanite.dashboard.service.UserService;
import it.eng.urbanite.dashboard.service.criteria.DashboardPageCriteria;
import it.eng.urbanite.dashboard.service.dto.DashboardPageCloningDTO;
import it.eng.urbanite.dashboard.service.dto.DashboardPageDTO;
import it.eng.urbanite.dashboard.utils.HeaderUtil;
import it.eng.urbanite.dashboard.web.error.BadRequestAlertException;
import tech.jhipster.service.filter.BooleanFilter;
import org.keycloak.representations.idm.UserRepresentation;

/**
 * REST controller for managing {@link it.eng.urbanite.dashboard.domain.DashboardPage}.
 */
@RestController
@CrossOrigin
@RequestMapping("/api")
public class DashboardPageResource {
    private final Logger log = LoggerFactory.getLogger(DashboardPageResource.class);
    private static final String ENTITY_NAME = "dashboardPage";

    private String applicationName = Constants.APP_NAME;

    private final DashboardPageService dashboardPageService;
    private final DashboardPageRepository dashboardPageRepository;
    private final DashboardPageQueryService dashboardPageQueryService;
    private final UserService userService;
    
    public DashboardPageResource(
        DashboardPageService dashboardPageService,
        DashboardPageRepository dashboardPageRepository,
        DashboardPageQueryService dashboardPageQueryService,
        UserService userService
    ) {
        this.dashboardPageService = dashboardPageService;
        this.dashboardPageRepository = dashboardPageRepository;
        this.dashboardPageQueryService = dashboardPageQueryService;
        this.userService = userService;
    }
    

    @PostMapping("/dashboard-pages")
    public ResponseEntity<DashboardPageDTO> createDashboardPage(@Valid @RequestBody DashboardPageDTO dashboardPageDTO)
        throws URISyntaxException {
        log.debug("REST request to save DashboardPage : {}", dashboardPageDTO);
        if (dashboardPageDTO.getId() != null) {
            throw new BadRequestAlertException("A new dashboardPage cannot already have an ID "+ ENTITY_NAME+ " idexists");
        }
        
        //Fill CreatedBy/CreatedDate
        dashboardPageDTO.setCreatedBy(SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM));
        dashboardPageDTO.setCreatedDate(Instant.now());
        
        DashboardPageDTO result = dashboardPageService.save(dashboardPageDTO);
        return ResponseEntity
            .created(new URI("/api/dashboard-pages/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    @PutMapping("/dashboard-pages/{id}")
    public ResponseEntity<DashboardPageDTO> updateDashboardPage(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody DashboardPageDTO dashboardPageDTO
    ) throws URISyntaxException {
        log.debug("REST request to update DashboardPage : {}, {}", id, dashboardPageDTO);
        if (dashboardPageDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id "+ ENTITY_NAME+ " idnull");
        }
        if (!Objects.equals(id, dashboardPageDTO.getId())) {
            throw new BadRequestAlertException("Invalid ID "+ ENTITY_NAME+ " idinvalid");
        }

        if (!dashboardPageRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found "+ ENTITY_NAME+ " idnotfound");
        }

        DashboardPageDTO result = dashboardPageService.save(dashboardPageDTO);
        return ResponseEntity.ok(result);
    }

    @GetMapping("/personal-dashboard-pages")
    public ResponseEntity<List<DashboardPageDTO>> getAllPersonalDashboardPages() {
    	log.info("REST request to get all PERSONAL DashboardPages");
    	String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
    	List<DashboardPageDTO> list = dashboardPageService.findAllPagesPersonal(createdBy);
    	return ResponseEntity.ok().body(list);
    }

    @GetMapping("/editable-dashboard-pages")
    public ResponseEntity<List<DashboardPageDTO>> getAllEditableDashboardPages() {
    	log.info("REST request to get all EDITABLE DashboardPages, (ONLY created-by owner)");
    	
    	//Add filter to retrieve only Pages created by user
    	DashboardPageCriteria criteria = new DashboardPageCriteria();
    	BooleanFilter filter = new BooleanFilter();
    	filter.setEquals(Boolean.TRUE);
		criteria.setMypages(filter);
		
        log.debug("REST request to get DashboardPages created by Owner by criteria: {}", criteria);
        List<DashboardPageDTO> list = dashboardPageQueryService.findByCriteria(criteria);
        log.debug("REST result for DashboardPages created by Owner : size is " +  list.size());
        
        return ResponseEntity.ok().body(list);
    }
    
    @GetMapping("/shared-dashboard-pages")
    public ResponseEntity<List<DashboardPageDTO>> getAllSharedDashboardPages() {
    	log.info("REST request to get all SHARED DashboardPages");
    	
    	String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
    	//List<String> roles = SecurityUtils.getCurrentRoleCodes();
    	List<String> roles = SecurityUtils.getUserRoles(); 
    	
    	//GROUPS
    	//List<String> groups = SecurityUtils.getCurrentGroupsCode(); 
		HashMap<String, List<UserRepresentation>> map = userService.findGroupsFromUser(createdBy);
		List<String> groups = (map!=null && !map.isEmpty()) ? new ArrayList<String>(map.keySet()): new ArrayList<>();
		
    	List<DashboardPageDTO> list = dashboardPageService.findAllPagesShared(createdBy, Boolean.TRUE, roles, groups);
    	return ResponseEntity.ok().body(list);
    }
    
    @GetMapping("/clonable-dashboard-pages")
    public ResponseEntity<List<DashboardPageDTO>> getAllClonableDashboardPages() {
    	log.info("###  REST request to get all Clonable DashboardPages, (PERSONAL created-by owner, and SHARED)");
    	String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
    	
    	//personal-dashboard-pages
    	List<DashboardPageDTO> listPersonal = dashboardPageService.findAllPagesPersonal(createdBy);
    	log.debug("Personal size is " +  listPersonal.size());

    	//shared-dashboard-pages
    	List<String> roles = SecurityUtils.getUserRoles(); 
    	HashMap<String, List<UserRepresentation>> map = userService.findGroupsFromUser(createdBy);
		List<String> groups = (map!=null && !map.isEmpty()) ? new ArrayList<String>(map.keySet()): new ArrayList<>();
		List<DashboardPageDTO> listShared = dashboardPageService.findAllPagesShared(createdBy, Boolean.TRUE, roles, groups);
		log.info("###  shared-dashboard-pages size is " +  listShared.size());
    	
		//UNIONS
		List<DashboardPageDTO> list = new ArrayList<>();
		list.addAll(listPersonal);
		list.addAll(listShared);
		log.info("### REST Total(shared+personal) size is " +  list.size());
		  
        return ResponseEntity.ok().body(list);
    }

    
    @GetMapping("/all-dashboard-pages")
    public ResponseEntity<List<DashboardPageDTO>> getAllDashboardPages() {
    	log.debug("REST request to get DashboardPages");
    	List<DashboardPageDTO> list = dashboardPageService.findAll();
    	return ResponseEntity.ok().body(list);
    }

    @GetMapping("/dashboard-pages/{id}")
    public ResponseEntity<DashboardPageDTO> getDashboardPage(@PathVariable Long id) {
        log.debug("REST request to get DashboardPage : {}", id);
        Optional<DashboardPageDTO> dashboardPageDTO = dashboardPageService.findOne(id);
        return ResponseEntity.ok().body(dashboardPageDTO.orElse(null));
    }

    @GetMapping("/duplicate-dashboard-pages/{id}")
    public ResponseEntity<DashboardPageDTO> getDuplicateDashboardPage(@PathVariable Long id) {
    	log.debug("REST request to duplicate DashboardPage : {}", id);
    	String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
    	
        DashboardPageDTO dashboardPageDTO = dashboardPageService.duplicatePage(id, createdBy, null, CloningOpionType.ALL);
    	return ResponseEntity.ok().body(dashboardPageDTO);
    }

    @PostMapping("/duplicate-by-params-dashboard-pages")
    public ResponseEntity<DashboardPageDTO> postDuplicateDashboardPage(@Valid @RequestBody DashboardPageCloningDTO pageCloningDTO){
    	DashboardPageDTO dashboardPageDTO = null;
    	try {
    		log.info("REST request to duplicate with params DashboardPage : {}, {}, {}", pageCloningDTO.getId(), pageCloningDTO.getDashboardName(), pageCloningDTO.getOption());
            if (pageCloningDTO.getId() == null) {
                throw new BadRequestAlertException("A new dashboardPage cannot be cloned due to id is NULL!");
            }
    		String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
    		dashboardPageDTO = dashboardPageService.duplicatePage(
    				pageCloningDTO.getId(), 
    				createdBy, 
    				pageCloningDTO.getDashboardName(), 
    				pageCloningDTO.getOption());
    	} catch (Exception e) {
    		e.printStackTrace();
    		log.error("Error in duplicate-by-params-dashboard due to error: {}", e.getMessage() + " - " + e.getCause());
    	}
    	log.info("DONE!! The duplicate has been successfully done with params DashboardPage {} {} {} ",pageCloningDTO.getId(), pageCloningDTO.getDashboardName(), pageCloningDTO.getOption());
        return ResponseEntity.ok().body(dashboardPageDTO);
    }

    @DeleteMapping("/dashboard-pages/{id}")
    public ResponseEntity<Void> deleteDashboardPage(@PathVariable Long id) {
        log.debug("REST request to delete DashboardPage : {}", id);
        dashboardPageService.delete(id);
        return ResponseEntity.noContent().build();
    }
    
    @DeleteMapping("/dashboard-pages-deep/{id}")
    public ResponseEntity<Void> deleteDashboardPageDeep(@PathVariable Long id) {
        log.debug("REST request to delete DashboardPage : {}", id);
        dashboardPageService.deepDelete(id);
        return ResponseEntity.noContent().build();
    }
    
}
