package it.eng.urbanite.dashboard.web;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.Instant;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.eng.urbanite.dashboard.config.Constants;
import it.eng.urbanite.dashboard.model.enumeration.MenuType;
import it.eng.urbanite.dashboard.repository.MenuBlockRepository;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.MenuBlockQueryService;
import it.eng.urbanite.dashboard.service.MenuBlockService;
import it.eng.urbanite.dashboard.service.dto.MenuBlockDTO;
import it.eng.urbanite.dashboard.web.error.BadRequestAlertException;
import tech.jhipster.web.util.HeaderUtil;

/**
 * REST controller for managing {@link it.eng.urbanite.dashboard.domain.MenuBlock}.
 */
@RestController
@CrossOrigin
@RequestMapping("/api")
public class MenuBlockResource {
	private final Logger log = LoggerFactory.getLogger(MenuBlockResource.class);
	private static final String ENTITY_NAME = "menuBlock";

	private String applicationName = Constants.APP_NAME;

	private final MenuBlockService menuBlockService;
	private final MenuBlockRepository menuBlockRepository;

	public MenuBlockResource(
			MenuBlockService menuBlockService,
			MenuBlockRepository menuBlockRepository,
			MenuBlockQueryService menuBlockQueryService
			) {
		this.menuBlockService = menuBlockService;
		this.menuBlockRepository = menuBlockRepository;
	}

	@PostMapping("/menu-blocks")
	public ResponseEntity<MenuBlockDTO> createMenuBlock(@Valid @RequestBody MenuBlockDTO menuBlockDTO) throws URISyntaxException {
		log.debug("REST request to save MenuBlock : {}", menuBlockDTO);
		if (menuBlockDTO.getId() != null) {
			throw new BadRequestAlertException("A new menuBlock cannot already have an ID "+ ENTITY_NAME+ " idexists");
		}

		//Fill CreatedBy/CreatedDate
		menuBlockDTO.setCreatedBy(SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM));
		menuBlockDTO.setCreatedDate(Instant.now());

		MenuBlockDTO result = menuBlockService.save(menuBlockDTO);
		return ResponseEntity
				.created(new URI("/api/menu-blocks/" + result.getId()))
				.headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
				.body(result);
	}

	@PutMapping("/menu-blocks/{id}")
	public ResponseEntity<MenuBlockDTO> updateMenuBlock(
			@PathVariable(value = "id", required = false) final Long id,
			@Valid @RequestBody MenuBlockDTO menuBlockDTO
			) throws URISyntaxException {
		log.debug("REST request to update MenuBlock : {}, {}", id, menuBlockDTO);
		if (menuBlockDTO.getId() == null) {
			throw new BadRequestAlertException("Invalid id "+ ENTITY_NAME+ " idnull");
		}
		if (!Objects.equals(id, menuBlockDTO.getId())) {
			throw new BadRequestAlertException("Invalid ID "+ ENTITY_NAME+ " idinvalid");
		}

		if (!menuBlockRepository.existsById(id)) {
			throw new BadRequestAlertException("Entity not found "+ ENTITY_NAME+ " idnotfound");
		}

		MenuBlockDTO result = menuBlockService.save(menuBlockDTO);
		return ResponseEntity.ok(result);
	}

	@GetMapping("/menu-blocks")
	public ResponseEntity<List<MenuBlockDTO>> getAllMenuBlocks() {
		log.debug("REST request to get MenuBlocks");
		List<MenuBlockDTO> list = menuBlockService.findAll();
		return ResponseEntity.ok().body(list);
	}

	
	@GetMapping("/linked-menu-blocks")
	public ResponseEntity<List<MenuBlockDTO>> getLinkedMenuBlocks() {
		log.debug("REST request to get Linked MenuBlocks");
		String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
		List<MenuBlockDTO> list = menuBlockService.findLinkedItems(null, createdBy);
		return ResponseEntity.ok().body(list);
	}
	
	@GetMapping("/personal-menu-blocks")
	public ResponseEntity<List<MenuBlockDTO>> getAllPersonalMenuBlocks() {
		log.debug("REST request to get Personal Linked MenuBlocks");
		String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
		List<MenuBlockDTO> list = menuBlockService.findLinkedItems(MenuType.PRIVATE, createdBy);
		return ResponseEntity.ok().body(list);
	}

	@GetMapping("/shared-menu-blocks")
	public ResponseEntity<List<MenuBlockDTO>> getAllSharedMenuBlocks() {
		log.debug("REST request to get Shared Linked MenuBlocks");
		String createdBy = SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM);
		List<MenuBlockDTO> list = menuBlockService.findLinkedItems(MenuType.SHARED, createdBy);
		return ResponseEntity.ok().body(list);
	}
	@GetMapping("/public-menu-blocks")
	public ResponseEntity<List<MenuBlockDTO>> getAllPublicMenuBlocks() {
		log.debug("REST request to get PUBLIC MenuBlocks");
        List<MenuBlockDTO> list = menuBlockService.findAll();
        return ResponseEntity.ok().body(list);
	}
	
	@GetMapping("/editable-menu-blocks")
	public ResponseEntity<List<MenuBlockDTO>> getAllEditableMenuBlocks() {
		log.info("REST request to get Editable (SHARED or PERSONAL created-by user) Linked MenuBlocks");
		log.info("Update: REST request to get SHARED, PERSONAL and PUBLIC FIXED menu blocks created by default by the system");
		List<MenuBlockDTO> list = menuBlockService.findEditableBlocks();
		return ResponseEntity.ok().body(list);
	}

	@GetMapping("/menu-blocks/{id}")
	public ResponseEntity<MenuBlockDTO> getMenuBlock(@PathVariable Long id) {
		log.debug("REST request to get MenuBlock : {}", id);
		Optional<MenuBlockDTO> menuBlockDTO = menuBlockService.findOne(id);
		return ResponseEntity.ok().body(menuBlockDTO.orElse(null));
	}
		
	@DeleteMapping("/menu-blocks/{id}")
	public ResponseEntity<Void> deleteMenuBlock(@PathVariable Long id) {
		log.debug("REST request to delete MenuBlock : {}", id);
		menuBlockService.delete(id);
		return ResponseEntity.noContent().build();
	}
}
