package it.eng.urbanite.dashboard.web;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.Instant;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.eng.urbanite.dashboard.config.Constants;
import it.eng.urbanite.dashboard.model.enumeration.MenuType;
import it.eng.urbanite.dashboard.repository.MenuItemRepository;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.MenuItemQueryService;
import it.eng.urbanite.dashboard.service.MenuItemService;
import it.eng.urbanite.dashboard.service.criteria.MenuItemCriteria;
import it.eng.urbanite.dashboard.service.criteria.MenuItemCriteria.MenuTypeFilter;
import it.eng.urbanite.dashboard.service.dto.MenuItemDTO;
import it.eng.urbanite.dashboard.web.error.BadRequestAlertException;
import tech.jhipster.service.filter.LongFilter;
import tech.jhipster.web.util.HeaderUtil;

/**
 * REST controller for managing {@link it.eng.urbanite.dashboard.domain.MenuItem}.
 */
@RestController
@CrossOrigin
@RequestMapping("/api")
public class MenuItemResource {
    private final Logger log = LoggerFactory.getLogger(MenuItemResource.class);
    private static final String ENTITY_NAME = "menuItem";

    private String applicationName = Constants.APP_NAME;

    private final MenuItemService menuItemService;
    private final MenuItemRepository menuItemRepository;
    private final MenuItemQueryService menuItemQueryService;

    public MenuItemResource(
        MenuItemService menuItemService,
        MenuItemRepository menuItemRepository,
        MenuItemQueryService menuItemQueryService
    ) {
        this.menuItemService = menuItemService;
        this.menuItemRepository = menuItemRepository;
        this.menuItemQueryService = menuItemQueryService;
    }

    @PostMapping("/menu-items")
    public ResponseEntity<MenuItemDTO> createMenuItem(@Valid @RequestBody MenuItemDTO menuItemDTO) throws URISyntaxException {
        log.debug("REST request to save MenuItem : {}", menuItemDTO);
        if (menuItemDTO.getId() != null) {
            throw new BadRequestAlertException("A new menuItem cannot already have an ID "+ ENTITY_NAME+ " idexists");
        }
        
        //Fill CreatedBy/CreatedDate
        menuItemDTO.setCreatedBy(SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM));
        menuItemDTO.setCreatedDate(Instant.now());
        
        MenuItemDTO result = menuItemService.save(menuItemDTO);
        return ResponseEntity
            .created(new URI("/api/menu-items/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    @PutMapping("/menu-items/{id}")
    public ResponseEntity<MenuItemDTO> updateMenuItem(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody MenuItemDTO menuItemDTO
    ) throws URISyntaxException {
        log.debug("REST request to update MenuItem : {}, {}", id, menuItemDTO);
        if (menuItemDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id "+ ENTITY_NAME+ " idnull");
        }
        if (!Objects.equals(id, menuItemDTO.getId())) {
            throw new BadRequestAlertException("Invalid ID "+ ENTITY_NAME+ " idinvalid");
        }

        if (!menuItemRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found "+ ENTITY_NAME+ " idnotfound");
        }

        MenuItemDTO result = menuItemService.save(menuItemDTO);
        return ResponseEntity.ok(result);
    }

    @GetMapping("/menu-items")
    public ResponseEntity<List<MenuItemDTO>> getAllMenuItems() {
        log.debug("REST request to get MenuItems");
        List<MenuItemDTO> list = menuItemService.findAll();
        return ResponseEntity.ok().body(list);
    }
    
    @GetMapping("/menu-items/{id}")
    public ResponseEntity<MenuItemDTO> getMenuItem(@PathVariable Long id) {
        log.debug("REST request to get MenuItem : {}", id);
        Optional<MenuItemDTO> menuItemDTO = menuItemService.findOne(id);
        return ResponseEntity.ok().body(menuItemDTO.orElse(null));
    }

    
	@GetMapping("/menu-items-bypage/{pageId}")
	public ResponseEntity<List<MenuItemDTO>> getMenuItemsByPageId(@PathVariable Long pageId) {
		MenuItemCriteria criteria = new MenuItemCriteria();
    	LongFilter filter = new LongFilter();
    	filter.setEquals(pageId);
    	criteria.setDashboardPageId(filter);
		
        log.debug("REST request to get MenuItem by criteria: {}", criteria);
        List<MenuItemDTO> list = menuItemQueryService.findByCriteria(criteria);
        log.debug("REST result for MenuItem: size is " +  list.size());
        
        return ResponseEntity.ok().body(list);
	}
	
	
    @DeleteMapping("/menu-items/{id}")
    public ResponseEntity<Void> deleteMenuItem(@PathVariable Long id) {
        log.debug("REST request to delete MenuItem : {}", id);
        menuItemService.delete(id);
        return ResponseEntity.noContent().build();
    }
    
    
	@GetMapping("/public-menu-item")
	public ResponseEntity<List<MenuItemDTO>> getPublicMenuItem() {
		MenuItemCriteria criteria = new MenuItemCriteria();
    	MenuTypeFilter filter = new MenuTypeFilter();
    	filter.setEquals(MenuType.SHARED);
    	criteria.setType(filter);
		System.out.println(criteria.toString());
        log.debug("REST request to get MenuItem by criteria: {}", criteria);
        List<MenuItemDTO> list = menuItemQueryService.findByCriteria(criteria);
        log.debug("REST result for MenuItem: size is " +  list.size());
        
        return ResponseEntity.ok().body(list);
	}
}
