package it.eng.urbanite.dashboard.web;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.Instant;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.eng.urbanite.dashboard.config.Constants;
import it.eng.urbanite.dashboard.model.enumeration.TargetType;
import it.eng.urbanite.dashboard.repository.TargetAvailableRepository;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.TargetAvailableQueryService;
import it.eng.urbanite.dashboard.service.TargetAvailableService;
import it.eng.urbanite.dashboard.service.dto.TargetAvailableDTO;
import it.eng.urbanite.dashboard.utils.HeaderUtil;
import it.eng.urbanite.dashboard.web.error.BadRequestAlertException;

/**
 * REST controller for managing {@link it.eng.urbanite.dashboard.domain.TargetAvailable}.
 */
@RestController
@CrossOrigin
@RequestMapping("/api")
public class TargetAvailableResource {
    private final Logger log = LoggerFactory.getLogger(TargetAvailableResource.class);
    private static final String ENTITY_NAME = "targetAvailable";

    private String applicationName = Constants.APP_NAME;

    private final TargetAvailableService service;
    private final TargetAvailableRepository repository;
    
    public TargetAvailableResource(TargetAvailableService service, TargetAvailableRepository repository, TargetAvailableQueryService queryService) {
        this.service = service;
        this.repository = repository;
    }

    @PostMapping("/target-availables")
    public ResponseEntity<TargetAvailableDTO> createTarget(@Valid @RequestBody TargetAvailableDTO targetDTO) throws URISyntaxException {
        log.debug("REST request to save TargetAvailable : {}", targetDTO);
        if (targetDTO.getId() != null) {
            throw new BadRequestAlertException("A new TargetAvailable cannot already have an ID "+ ENTITY_NAME+ " idexists");
        }
        
        //Fill CreatedBy/CreatedDate
        targetDTO.setCreatedBy(SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM));
        targetDTO.setCreatedDate(Instant.now());
        
        TargetAvailableDTO result = service.save(targetDTO);
        return ResponseEntity
            .created(new URI("/api/target-availables/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    @PutMapping("/target-availables/{id}")
    public ResponseEntity<TargetAvailableDTO> updateTarget(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody TargetAvailableDTO targetDTO
    ) throws URISyntaxException {
        log.debug("REST request to update TargetAvailable : {}, {}", id, targetDTO);
        if (targetDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id "+ ENTITY_NAME+ " idnull");
        }
        if (!Objects.equals(id, targetDTO.getId())) {
            throw new BadRequestAlertException("Invalid ID "+ ENTITY_NAME+ " idinvalid");
        }

        if (!repository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found "+ ENTITY_NAME+ " idnotfound");
        }

        TargetAvailableDTO result = service.save(targetDTO);
        return ResponseEntity.ok(result);
    }

    @GetMapping("/target-availables")
    public ResponseEntity<List<TargetAvailableDTO>> getAllTargets() {
        log.debug("REST request to get TargetAvailables");
        List<TargetAvailableDTO> list = service.findAll();
        return ResponseEntity.ok().body(list);
    }

    @GetMapping("/target-availables-bytype/{type}")
    public ResponseEntity<List<TargetAvailableDTO>> getAllPersonTargets(@PathVariable String type) {
    	 log.debug("REST request to get TargetAvailables by type {}", type);
    	 try {
    		 TargetType curType = TargetType.forCode(type);
    		 List<TargetAvailableDTO> list = service.findByType(curType);
    		 return ResponseEntity.ok().body(list);
    	 } catch (Exception e) {
    		 e.printStackTrace();
    		 log.error("Unsupported type {}. Supported values are: {}", type, TargetType.getValues());
    	 }
         return ResponseEntity.badRequest().body(null);
    }
    
    @GetMapping("/target-availables/{id}")
    public ResponseEntity<TargetAvailableDTO> getTarget(@PathVariable Long id) {
        log.debug("REST request to get TargetAvailable : {}", id);
        Optional<TargetAvailableDTO> targetDTO = service.findOne(id);
        return ResponseEntity.ok().body(targetDTO.orElse(null));
    }

    @DeleteMapping("/target-availables/{id}")
    public ResponseEntity<Void> deleteTarget(@PathVariable Long id) {
        log.debug("REST request to delete TargetAvailable : {}", id);
        service.delete(id);
        return ResponseEntity.noContent().build();
    }
    
}
