package it.eng.urbanite.dashboard.web;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.Instant;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.eng.urbanite.dashboard.config.Constants;
import it.eng.urbanite.dashboard.model.enumeration.TargetType;
import it.eng.urbanite.dashboard.repository.TargetRepository;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.TargetQueryService;
import it.eng.urbanite.dashboard.service.TargetService;
import it.eng.urbanite.dashboard.service.criteria.TargetCriteria;
import it.eng.urbanite.dashboard.service.criteria.TargetCriteria.TargetTypeFilter;
import it.eng.urbanite.dashboard.service.dto.TargetDTO;
import it.eng.urbanite.dashboard.utils.HeaderUtil;
import it.eng.urbanite.dashboard.web.error.BadRequestAlertException;
import tech.jhipster.service.filter.LongFilter;
import tech.jhipster.service.filter.StringFilter;

/**
 * REST controller for managing {@link it.eng.urbanite.dashboard.domain.Target}.
 */
@RestController
@CrossOrigin
@RequestMapping("/api")
public class TargetResource {
    private final Logger log = LoggerFactory.getLogger(TargetResource.class);
    private static final String ENTITY_NAME = "target";

    private String applicationName = Constants.APP_NAME;

    private final TargetService targetService;
    private final TargetRepository targetRepository;
    private final TargetQueryService targetQueryService;
    
    public TargetResource(TargetService targetService, TargetRepository targetRepository, TargetQueryService targetQueryService) {
        this.targetService = targetService;
        this.targetRepository = targetRepository;
        this.targetQueryService = targetQueryService;
    }

    @PostMapping("/targets")
    public ResponseEntity<TargetDTO> createTarget(@Valid @RequestBody TargetDTO targetDTO) throws URISyntaxException {
        log.debug("REST request to save Target : {}", targetDTO);
        if (targetDTO.getId() != null) {
            throw new BadRequestAlertException("A new target cannot already have an ID "+ ENTITY_NAME+ " idexists");
        }
        
        //Fill CreatedBy/CreatedDate
        targetDTO.setCreatedBy(SecurityUtils.getCurrentUserLogin().orElse(Constants.SYSTEM));
        targetDTO.setCreatedDate(Instant.now());
        
        TargetDTO result = targetService.save(targetDTO);
        return ResponseEntity
            .created(new URI("/api/targets/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    @PutMapping("/targets/{id}")
    public ResponseEntity<TargetDTO> updateTarget(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody TargetDTO targetDTO
    ) throws URISyntaxException {
        log.debug("REST request to update Target : {}, {}", id, targetDTO);
        if (targetDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id "+ ENTITY_NAME+ " idnull");
        }
        if (!Objects.equals(id, targetDTO.getId())) {
            throw new BadRequestAlertException("Invalid ID "+ ENTITY_NAME+ " idinvalid");
        }

        if (!targetRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found "+ ENTITY_NAME+ " idnotfound");
        }

        TargetDTO result = targetService.save(targetDTO);
        return ResponseEntity.ok(result);
    }

    @GetMapping("/targets")
    public ResponseEntity<List<TargetDTO>> getAllTargets() {
        log.debug("REST request to get Targets");
        List<TargetDTO> list = targetService.findAll();
        return ResponseEntity.ok().body(list);
    }
    
    @GetMapping("/targets-bypage/{pageId}")
    public ResponseEntity<List<TargetDTO>> getAllTargetsByPageId(@PathVariable Long pageId) {
    	TargetCriteria criteria = new TargetCriteria();
    	LongFilter filter = new LongFilter();
    	filter.setEquals(pageId);
    	criteria.setDashboardPageId(filter);
		
        log.debug("REST request to get Targets by criteria: {}", criteria);
        List<TargetDTO> list = targetQueryService.findByCriteria(criteria);
        log.debug("REST result for Targets: size is " +  list.size());
        
        return ResponseEntity.ok().body(list);
    }
    
    @GetMapping("/targets-bytype/{type}")
    public ResponseEntity<List<TargetDTO>> getTargetByType(@PathVariable TargetType type) {
    	TargetCriteria criteria = new TargetCriteria();
    	TargetTypeFilter filter = new TargetTypeFilter();
    	filter.setEquals(type);
    	criteria.setType(new TargetTypeFilter(filter));
		
        log.debug("REST request to get Targets by criteria: {}", criteria);
        List<TargetDTO> list = targetQueryService.findByCriteria(criteria);
        log.debug("REST result for Targets: size is " +  list.size());
        
        return ResponseEntity.ok().body(list);
    }

    @GetMapping("/targets/{id}")
    public ResponseEntity<TargetDTO> getTarget(@PathVariable Long id) {
        log.debug("REST request to get Target : {}", id);
        Optional<TargetDTO> targetDTO = targetService.findOne(id);
        return ResponseEntity.ok().body(targetDTO.orElse(null));
    }

    @DeleteMapping("/targets/{id}")
    public ResponseEntity<Void> deleteTarget(@PathVariable Long id) {
        log.debug("REST request to delete Target : {}", id);
        targetService.delete(id);
        return ResponseEntity.noContent().build();
    }
    
}
