package it.eng.urbanite.dashboard.web;

import java.util.HashMap;
import java.util.List;

import org.keycloak.representations.idm.GroupRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.keycloak.representations.idm.UserRepresentation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.eng.urbanite.dashboard.model.enumeration.TargetType;
import it.eng.urbanite.dashboard.security.SecurityUtils;
import it.eng.urbanite.dashboard.service.TargetAvailableService;
import it.eng.urbanite.dashboard.service.UserService;
import it.eng.urbanite.dashboard.service.dto.TargetAvailableDTO;
import it.eng.urbanite.dashboard.service.dto.UserDTO;

/**
 * REST controller for managing Authentication (USERS, GROUPS, ROLES)
 */
@RestController
@CrossOrigin
@RequestMapping("/user")
public class UserResource {
	private final Logger log = LoggerFactory.getLogger(UserResource.class);
	private final UserService userService;
	private final TargetAvailableService service;

	
	public UserResource( UserService userService, TargetAvailableService service) {
		this.userService = userService;
		this.service = service;
	}

	/**
	 * Return list of IDM Users. 
	 * 
	 * Keycloak Note: Remember to enable for each specific "user" on TAB RoleMappings 
	 * the ClientRoles: "realm-management" adding the permission of "view-users". See picture "idm-config.png"
	 * 
	 * @return List of UserDTO
	 */
	@GetMapping("/get-idm-users")
	public ResponseEntity<List<UserDTO>> getIdmUsers() {
		log.debug("REST request to get-idm-users");
		List<UserDTO> list = userService.findAllUsers();
		return ResponseEntity.ok().body(list);
	}

	/**
	 * Return list of IDM Roles. 
	 * 
	 * Keycloak Note: Remember to enable for each specific "user" on TAB RoleMappings 
	 * the ClientRoles: "realm-management" adding the permission of "view-users". See picture "idm-config.png"
	 * 
	 * @return List of RoleRepresentation
	 */
	@GetMapping("/get-idm-roles")
	public ResponseEntity<List<RoleRepresentation>> getIdmRoles() {
		log.debug("REST request to get-idm-roles");
		List<RoleRepresentation> list = userService.findAllRoles();
		return ResponseEntity.ok().body(list);
	}

	/**
	 * Return list of IDM Groups. 
	 * 
	 * Keycloak Note: Remember to enable for each specific "user" on TAB RoleMappings 
	 * the ClientRoles: "realm-management" adding the permission of "view-users". See picture "idm-config.png"
	 * 
	 * @return List of GroupRepresentation
	 */
	@GetMapping("/get-idm-groups")
	public ResponseEntity<List<GroupRepresentation>> getIdmGroups() {
		log.debug("REST request to get-idm-groups");
		List<GroupRepresentation> list = userService.findAllGroups();
		return ResponseEntity.ok().body(list);
	}

	/**
	 * Return list of Roles for the specific logged user.
	 * @return List of String
	 */
	@GetMapping("/get-user-roles")
	public ResponseEntity<List<String>> getUserRoles() {
		log.info("REST request to get user-roles for the logged user...");
		List<String> list = SecurityUtils.getUserRoles();
		return ResponseEntity.ok().body(list);
	}
	
	/**
	 * Return list of Roles for the specific logged user.
	 * @return List of String
	 */
	@GetMapping("/get-user-groups/{username}")
	public ResponseEntity<HashMap<String, List<UserRepresentation>>> getUserGroups(@PathVariable String username) {
		log.info("REST request to get user-groups for username {} ...", username);
		HashMap<String, List<UserRepresentation>> map = userService.findGroupsFromUser(username);
		return ResponseEntity.ok().body(map);
	}
	
	
	
	
	/**
	 * Return users from the internal table called "TargetAvailable" with TargetType.PERSON
	 * @return List of TargetAvailableDTO
	 */
	@GetMapping("/get-users")
	public ResponseEntity<List<TargetAvailableDTO>> getUsers() {
		log.debug("REST request to get-users");
		List<TargetAvailableDTO> list = service.findByType(TargetType.PERSON);
		return ResponseEntity.ok().body(list);
	}
	
	/**
	 * Return roles from the internal table called "TargetAvailable" with TargetType.ROLE
	 * @return List of TargetAvailableDTO
	 */
	@GetMapping("/get-roles")
	public ResponseEntity<List<TargetAvailableDTO>> getRoles() {
		log.debug("REST request to get-roles");
		List<TargetAvailableDTO> list = service.findByType(TargetType.ROLE);
		return ResponseEntity.ok().body(list);
	}

	/**
	 * Return groups from the internal table called "TargetAvailable" with TargetType.GROUP
	 * @return List of TargetAvailableDTO
	 */
	@GetMapping("/get-groups")
	public ResponseEntity<List<TargetAvailableDTO>> getGroups() {
		log.debug("REST request to get-groups");
		List<TargetAvailableDTO> list = service.findByType(TargetType.GROUP);
		return ResponseEntity.ok().body(list);
	}

}
